@@ -36,10 +36,6 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
@@ -48,14 +44,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <locale.h>
 #include <errno.h>
 
-#ifdef SUPPORT_LIBREADLINE
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <readline/readline.h>
-#include <readline/history.h>
-#endif
-
 
 /* A number of things vary for Windows builds. Originally, pcretest opened its
 input and output without "b"; then I was told that "b" was needed in some
@@ -71,14 +59,6 @@ input mode under Windows. */
 #define INPUT_MODE   "r"
 #define OUTPUT_MODE  "wb"
 
-#ifndef isatty
-#define isatty _isatty         /* This is what Windows calls them, I'm told, */
-#endif                         /* though in some environments they seem to   */
-                               /* be already defined, hence the #ifndefs.    */
-#ifndef fileno
-#define fileno _fileno
-#endif
-
 #else
 #include <sys/time.h>          /* These two includes are needed */
 #include <sys/resource.h>      /* for setrlimit(). */
@@ -99,11 +79,10 @@ appropriately for an application, not for building PCRE. */
 #include "pcre.h"
 #include "pcre_internal.h"
 
-/* We need access to some of the data tables that PCRE uses. So as not to have
-to keep two copies, we include the source file here, changing the names of the
-external symbols to prevent clashes. */
+/* We need access to the data tables that PCRE uses. So as not to have to keep
+two copies, we include the source file here, changing the names of the external
+symbols to prevent clashes. */
 
-#define _pcre_ucp_gentype      ucp_gentype
 #define _pcre_utf8_table1      utf8_table1
 #define _pcre_utf8_table1_size utf8_table1_size
 #define _pcre_utf8_table2      utf8_table2
@@ -111,27 +90,25 @@ external symbols to prevent clashes. */
 #define _pcre_utf8_table4      utf8_table4
 #define _pcre_utt              utt
 #define _pcre_utt_size         utt_size
-#define _pcre_utt_names        utt_names
 #define _pcre_OP_lengths       OP_lengths
 
 #include "pcre_tables.c"
 
 /* We also need the pcre_printint() function for printing out compiled
 patterns. This function is in a separate file so that it can be included in
-pcre_compile.c when that module is compiled with debugging enabled. It needs to
-know which case is being compiled. */
+pcre_compile.c when that module is compiled with debugging enabled.
+
+The definition of the macro PRINTABLE, which determines whether to print an
+output character as-is or as a hex value when showing compiled patterns, is
+contained in this file. We uses it here also, in cases when the locale has not
+been explicitly changed, so as to get consistent output from systems that
+differ in their output from isprint() even in the "C" locale. */
 
-#define COMPILING_PCRETEST
 #include "pcre_printint.src"
 
-/* The definition of the macro PRINTABLE, which determines whether to print an
-output character as-is or as a hex value when showing compiled patterns, is
-contained in the printint.src file. We uses it here also, in cases when the
-locale has not been explicitly changed, so as to get consistent output from
-systems that differ in their output from isprint() even in the "C" locale. */
-
 #define PRINTHEX(c) (locale_set? isprint(c) : PRINTABLE(c))
 
+
 /* It is possible to compile this test program without including support for
 testing the POSIX interface, though this is not available via the standard
 Makefile. */
@@ -175,7 +152,6 @@ static int callout_count;
 static int callout_extra;
 static int callout_fail_count;
 static int callout_fail_id;
-static int debug_lengths;
 static int first_callout;
 static int locale_set = 0;
 static int show_malloc;
@@ -207,7 +183,6 @@ optimal way of handling this, but hey, this is just a test program!
 Arguments:
   f            the file to read
   start        where in buffer to start (this *must* be within buffer)
-  prompt       for stdin or readline()
 
 Returns:       pointer to the start of new data
                could be a copy of start, or could be moved
@@ -215,7 +190,7 @@ Returns:       pointer to the start of new data
 */
 
 static uschar *
-extend_inputline(FILE *f, uschar *start, const char *prompt)
+extend_inputline(FILE *f, uschar *start)
 {
 uschar *here = start;
 
@@ -226,36 +201,8 @@ for (;;)
   if (rlen > 1000)
     {
     int dlen;
-
-    /* If libreadline support is required, use readline() to read a line if the
-    input is a terminal. Note that readline() removes the trailing newline, so
-    we must put it back again, to be compatible with fgets(). */
-
-#ifdef SUPPORT_LIBREADLINE
-    if (isatty(fileno(f)))
-      {
-      size_t len;
-      char *s = readline(prompt);
-      if (s == NULL) return (here == start)? NULL : start;
-      len = strlen(s);
-      if (len > 0) add_history(s);
-      if (len > rlen - 1) len = rlen - 1;
-      memcpy(here, s, len);
-      here[len] = '\n';
-      here[len+1] = 0;
-      free(s);
-      }
-    else
-#endif
-
-    /* Read the next line by normal means, prompting if the file is stdin. */
-
-      {
-      if (f == stdin) printf(prompt);
-      if (fgets((char *)here, rlen,  f) == NULL)
-        return (here == start)? NULL : start;
-      }
-
+    if (fgets((char *)here, rlen,  f) == NULL)
+      return (here == start)? NULL : start;
     dlen = (int)strlen((char *)here);
     if (dlen > 0 && here[dlen - 1] == '\n') return start;
     here += dlen;
@@ -711,32 +658,6 @@ return count;
 
 
 
-/*************************************************
-*         Case-independent strncmp() function    *
-*************************************************/
-
-/*
-Arguments:
-  s         first string
-  t         second string
-  n         number of characters to compare
-
-Returns:    < 0, = 0, or > 0, according to the comparison
-*/
-
-static int
-strncmpic(uschar *s, uschar *t, int n)
-{
-while (n--)
-  {
-  int c = tolower(*s++) - tolower(*t++);
-  if (c) return c;
-  }
-return 0;
-}
-
-
-
 /*************************************************
 *         Check newline indicator                *
 *************************************************/
@@ -755,13 +676,11 @@ Returns:      appropriate PCRE_NEWLINE_xxx flags, or 0
 static int
 check_newline(uschar *p, FILE *f)
 {
-if (strncmpic(p, (uschar *)"cr>", 3) == 0) return PCRE_NEWLINE_CR;
-if (strncmpic(p, (uschar *)"lf>", 3) == 0) return PCRE_NEWLINE_LF;
-if (strncmpic(p, (uschar *)"crlf>", 5) == 0) return PCRE_NEWLINE_CRLF;
-if (strncmpic(p, (uschar *)"anycrlf>", 8) == 0) return PCRE_NEWLINE_ANYCRLF;
-if (strncmpic(p, (uschar *)"any>", 4) == 0) return PCRE_NEWLINE_ANY;
-if (strncmpic(p, (uschar *)"bsr_anycrlf>", 12) == 0) return PCRE_BSR_ANYCRLF;
-if (strncmpic(p, (uschar *)"bsr_unicode>", 12) == 0) return PCRE_BSR_UNICODE;
+if (strncmp((char *)p, "cr>", 3) == 0) return PCRE_NEWLINE_CR;
+if (strncmp((char *)p, "lf>", 3) == 0) return PCRE_NEWLINE_LF;
+if (strncmp((char *)p, "crlf>", 5) == 0) return PCRE_NEWLINE_CRLF;
+if (strncmp((char *)p, "anycrlf>", 8) == 0) return PCRE_NEWLINE_ANYCRLF;
+if (strncmp((char *)p, "any>", 4) == 0) return PCRE_NEWLINE_ANY;
 fprintf(f, "Unknown newline type at: <%s\n", p);
 return 0;
 }
@@ -775,14 +694,7 @@ return 0;
 static void
 usage(void)
 {
-printf("Usage:     pcretest [options] [<input file> [<output file>]]\n\n");
-printf("Input and output default to stdin and stdout.\n");
-#ifdef SUPPORT_LIBREADLINE
-printf("If input is a terminal, readline() is used to read from it.\n");
-#else
-printf("This version of pcretest is not linked with readline().\n");
-#endif
-printf("\nOptions:\n");
+printf("Usage:     pcretest [options] [<input> [<output>]]\n");
 printf("  -b       show compiled code (bytecode)\n");
 printf("  -C       show PCRE compile-time options and exit\n");
 printf("  -d       debug: show compiled code and information (-b and -i)\n");
@@ -791,7 +703,6 @@ printf("  -dfa     force DFA matching for all subjects\n");
 #endif
 printf("  -help    show usage information\n");
 printf("  -i       show information about compiled patterns\n"
-       "  -M       find MATCH_LIMIT minimum for each subject\n"
        "  -m       output memory used information\n"
        "  -o <n>   set size of offsets vector to <n>\n");
 #if !defined NOPOSIX
@@ -821,7 +732,6 @@ int main(int argc, char **argv)
 FILE *infile = stdin;
 int options = 0;
 int study_options = 0;
-int default_find_match_limit = FALSE;
 int op = 1;
 int timeit = 0;
 int timeitm = 0;
@@ -881,7 +791,6 @@ while (argc > 1 && argv[op][0] == '-')
   else if (strcmp(argv[op], "-b") == 0) debug = 1;
   else if (strcmp(argv[op], "-i") == 0) showinfo = 1;
   else if (strcmp(argv[op], "-d") == 0) showinfo = debug = 1;
-  else if (strcmp(argv[op], "-M") == 0) default_find_match_limit = TRUE;
 #if !defined NODFA
   else if (strcmp(argv[op], "-dfa") == 0) all_use_dfa = 1;
 #endif
@@ -934,7 +843,6 @@ while (argc > 1 && argv[op][0] == '-')
   else if (strcmp(argv[op], "-C") == 0)
     {
     int rc;
-    unsigned long int lrc;
     printf("PCRE version %s\n", pcre_version());
     printf("Compiled with\n");
     (void)pcre_config(PCRE_CONFIG_UTF8, &rc);
@@ -942,23 +850,18 @@ while (argc > 1 && argv[op][0] == '-')
     (void)pcre_config(PCRE_CONFIG_UNICODE_PROPERTIES, &rc);
     printf("  %sUnicode properties support\n", rc? "" : "No ");
     (void)pcre_config(PCRE_CONFIG_NEWLINE, &rc);
-    /* Note that these values are always the ASCII values, even
-    in EBCDIC environments. CR is 13 and NL is 10. */
-    printf("  Newline sequence is %s\n", (rc == 13)? "CR" :
-      (rc == 10)? "LF" : (rc == (13<<8 | 10))? "CRLF" :
+    printf("  Newline sequence is %s\n", (rc == '\r')? "CR" :
+      (rc == '\n')? "LF" : (rc == ('\r'<<8 | '\n'))? "CRLF" :
       (rc == -2)? "ANYCRLF" :
       (rc == -1)? "ANY" : "???");
-    (void)pcre_config(PCRE_CONFIG_BSR, &rc);
-    printf("  \\R matches %s\n", rc? "CR, LF, or CRLF only" :
-                                     "all Unicode newlines");
     (void)pcre_config(PCRE_CONFIG_LINK_SIZE, &rc);
     printf("  Internal link size = %d\n", rc);
     (void)pcre_config(PCRE_CONFIG_POSIX_MALLOC_THRESHOLD, &rc);
     printf("  POSIX malloc threshold = %d\n", rc);
-    (void)pcre_config(PCRE_CONFIG_MATCH_LIMIT, &lrc);
-    printf("  Default match limit = %ld\n", lrc);
-    (void)pcre_config(PCRE_CONFIG_MATCH_LIMIT_RECURSION, &lrc);
-    printf("  Default recursion depth limit = %ld\n", lrc);
+    (void)pcre_config(PCRE_CONFIG_MATCH_LIMIT, &rc);
+    printf("  Default match limit = %d\n", rc);
+    (void)pcre_config(PCRE_CONFIG_MATCH_LIMIT_RECURSION, &rc);
+    printf("  Default recursion depth limit = %d\n", rc);
     (void)pcre_config(PCRE_CONFIG_STACKRECURSE, &rc);
     printf("  Match recursion uses %s\n", rc? "stack" : "heap");
     goto EXIT;
@@ -1047,6 +950,7 @@ while (!done)
   size_t size, regex_gotten_store;
   int do_study = 0;
   int do_debug = debug;
+  int debug_lengths = 1;
   int do_G = 0;
   int do_g = 0;
   int do_showinfo = showinfo;
@@ -1055,9 +959,9 @@ while (!done)
   int erroroffset, len, delimiter, poffset;
 
   use_utf8 = 0;
-  debug_lengths = 1;
 
-  if (extend_inputline(infile, buffer, "  re> ") == NULL) break;
+  if (infile == stdin) printf("  re> ");
+  if (extend_inputline(infile, buffer) == NULL) break;
   if (infile != stdin) fprintf(outfile, "%s", (char *)buffer);
   fflush(outfile);
 
@@ -1157,7 +1061,7 @@ while (!done)
 
   if (isalnum(delimiter) || delimiter == '\\')
     {
-    fprintf(outfile, "** Delimiter must not be alphanumeric or \\\n");
+    fprintf(outfile, "** Delimiter must not be alphameric or \\\n");
     goto SKIP_DATA;
     }
 
@@ -1173,7 +1077,8 @@ while (!done)
       pp++;
       }
     if (*pp != 0) break;
-    if ((pp = extend_inputline(infile, pp, "    > ")) == NULL)
+    if (infile == stdin) printf("    > ");
+    if ((pp = extend_inputline(infile, pp)) == NULL)
       {
       fprintf(outfile, "** Unexpected EOF\n");
       done = 1;
@@ -1265,18 +1170,10 @@ while (!done)
 
       case '<':
         {
-        if (strncmp((char *)pp, "JS>", 3) == 0)
-          {
-          options |= PCRE_JAVASCRIPT_COMPAT;
-          pp += 3;
-          }
-        else
-          {
-          int x = check_newline(pp, outfile);
-          if (x == 0) goto SKIP_DATA;
-          options |= x;
-          while (*pp++ != '>');
-          }
+        int x = check_newline(pp, outfile);
+        if (x == 0) goto SKIP_DATA;
+        options |= x;
+        while (*pp++ != '>');
         }
       break;
 
@@ -1306,7 +1203,6 @@ while (!done)
     if ((options & PCRE_DOTALL) != 0) cflags |= REG_DOTALL;
     if ((options & PCRE_NO_AUTO_CAPTURE) != 0) cflags |= REG_NOSUB;
     if ((options & PCRE_UTF8) != 0) cflags |= REG_UTF8;
-    if ((options & PCRE_UNGREEDY) != 0) cflags |= REG_UNGREEDY;
 
     rc = regcomp(&preg, (char *)p, cflags);
 
@@ -1327,8 +1223,6 @@ while (!done)
 #endif  /* !defined NOPOSIX */
 
     {
-    unsigned long int get_options;
-
     if (timeit > 0)
       {
       register int i;
@@ -1358,7 +1252,7 @@ while (!done)
         {
         for (;;)
           {
-          if (extend_inputline(infile, buffer, NULL) == NULL)
+          if (extend_inputline(infile, buffer) == NULL)
             {
             done = 1;
             goto CONTINUE;
@@ -1372,16 +1266,9 @@ while (!done)
       goto CONTINUE;
       }
 
-    /* Compilation succeeded. It is now possible to set the UTF-8 option from
-    within the regex; check for this so that we know how to process the data
-    lines. */
-
-    new_info(re, NULL, PCRE_INFO_OPTIONS, &get_options);
-    if ((get_options & PCRE_UTF8) != 0) use_utf8 = 1;
-
-    /* Print information if required. There are now two info-returning
-    functions. The old one has a limited interface and returns only limited
-    data. Check that it agrees with the newer one. */
+    /* Compilation succeeded; print data if required. There are now two
+    info-returning functions. The old one has a limited interface and
+    returns only limited data. Check that it agrees with the newer one. */
 
     if (log_store)
       fprintf(outfile, "Memory allocation (code space): %d\n",
@@ -1428,32 +1315,24 @@ while (!done)
     if (do_flip)
       {
       real_pcre *rre = (real_pcre *)re;
-      rre->magic_number =
-        byteflip(rre->magic_number, sizeof(rre->magic_number));
+      rre->magic_number = byteflip(rre->magic_number, sizeof(rre->magic_number));
       rre->size = byteflip(rre->size, sizeof(rre->size));
       rre->options = byteflip(rre->options, sizeof(rre->options));
-      rre->flags = (pcre_uint16)byteflip(rre->flags, sizeof(rre->flags));
-      rre->top_bracket =
-        (pcre_uint16)byteflip(rre->top_bracket, sizeof(rre->top_bracket));
-      rre->top_backref =
-        (pcre_uint16)byteflip(rre->top_backref, sizeof(rre->top_backref));
-      rre->first_byte =
-        (pcre_uint16)byteflip(rre->first_byte, sizeof(rre->first_byte));
-      rre->req_byte =
-        (pcre_uint16)byteflip(rre->req_byte, sizeof(rre->req_byte));
-      rre->name_table_offset = (pcre_uint16)byteflip(rre->name_table_offset,
+      rre->top_bracket = byteflip(rre->top_bracket, sizeof(rre->top_bracket));
+      rre->top_backref = byteflip(rre->top_backref, sizeof(rre->top_backref));
+      rre->first_byte = byteflip(rre->first_byte, sizeof(rre->first_byte));
+      rre->req_byte = byteflip(rre->req_byte, sizeof(rre->req_byte));
+      rre->name_table_offset = byteflip(rre->name_table_offset,
         sizeof(rre->name_table_offset));
-      rre->name_entry_size = (pcre_uint16)byteflip(rre->name_entry_size,
+      rre->name_entry_size = byteflip(rre->name_entry_size,
         sizeof(rre->name_entry_size));
-      rre->name_count = (pcre_uint16)byteflip(rre->name_count,
-        sizeof(rre->name_count));
+      rre->name_count = byteflip(rre->name_count, sizeof(rre->name_count));
 
       if (extra != NULL)
         {
         pcre_study_data *rsd = (pcre_study_data *)(extra->study_data);
         rsd->size = byteflip(rsd->size, sizeof(rsd->size));
-        rsd->flags = byteflip(rsd->flags, sizeof(rsd->flags));
-        rsd->minlength = byteflip(rsd->minlength, sizeof(rsd->minlength));
+        rsd->options = byteflip(rsd->options, sizeof(rsd->options));
         }
       }
 
@@ -1467,19 +1346,17 @@ while (!done)
       pcre_printint(re, outfile, debug_lengths);
       }
 
-    /* We already have the options in get_options (see above) */
-
     if (do_showinfo)
       {
-      unsigned long int all_options;
+      unsigned long int get_options, all_options;
 #if !defined NOINFOCHECK
       int old_first_char, old_options, old_count;
 #endif
-      int count, backrefmax, first_char, need_char, okpartial, jchanged,
-        hascrorlf;
+      int count, backrefmax, first_char, need_char, okpartial, jchanged;
       int nameentrysize, namecount;
       const uschar *nametable;
 
+      new_info(re, NULL, PCRE_INFO_OPTIONS, &get_options);
       new_info(re, NULL, PCRE_INFO_SIZE, &size);
       new_info(re, NULL, PCRE_INFO_CAPTURECOUNT, &count);
       new_info(re, NULL, PCRE_INFO_BACKREFMAX, &backrefmax);
@@ -1490,7 +1367,6 @@ while (!done)
       new_info(re, NULL, PCRE_INFO_NAMETABLE, (void *)&nametable);
       new_info(re, NULL, PCRE_INFO_OKPARTIAL, &okpartial);
       new_info(re, NULL, PCRE_INFO_JCHANGED, &jchanged);
-      new_info(re, NULL, PCRE_INFO_HASCRORLF, &hascrorlf);
 
 #if !defined NOINFOCHECK
       old_count = pcre_info(re, &old_options, &old_first_char);
@@ -1533,21 +1409,18 @@ while (!done)
         }
 
       if (!okpartial) fprintf(outfile, "Partial matching not supported\n");
-      if (hascrorlf) fprintf(outfile, "Contains explicit CR or LF match\n");
 
       all_options = ((real_pcre *)re)->options;
       if (do_flip) all_options = byteflip(all_options, sizeof(all_options));
 
       if (get_options == 0) fprintf(outfile, "No options\n");
-        else fprintf(outfile, "Options:%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s\n",
+        else fprintf(outfile, "Options:%s%s%s%s%s%s%s%s%s%s%s%s%s\n",
           ((get_options & PCRE_ANCHORED) != 0)? " anchored" : "",
           ((get_options & PCRE_CASELESS) != 0)? " caseless" : "",
           ((get_options & PCRE_EXTENDED) != 0)? " extended" : "",
           ((get_options & PCRE_MULTILINE) != 0)? " multiline" : "",
           ((get_options & PCRE_FIRSTLINE) != 0)? " firstline" : "",
           ((get_options & PCRE_DOTALL) != 0)? " dotall" : "",
-          ((get_options & PCRE_BSR_ANYCRLF) != 0)? " bsr_anycrlf" : "",
-          ((get_options & PCRE_BSR_UNICODE) != 0)? " bsr_unicode" : "",
           ((get_options & PCRE_DOLLAR_ENDONLY) != 0)? " dollar_endonly" : "",
           ((get_options & PCRE_EXTRA) != 0)? " extra" : "",
           ((get_options & PCRE_UNGREEDY) != 0)? " ungreedy" : "",
@@ -1630,14 +1503,10 @@ while (!done)
         else
           {
           uschar *start_bits = NULL;
-          int minlength;
-
-          new_info(re, extra, PCRE_INFO_MINLENGTH, &minlength);
-          fprintf(outfile, "Subject length lower bound = %d\n", minlength);
-
           new_info(re, extra, PCRE_INFO_FIRSTTABLE, &start_bits);
+
           if (start_bits == NULL)
-            fprintf(outfile, "No set of starting bytes\n");
+            fprintf(outfile, "No starting byte set\n");
           else
             {
             int i;
@@ -1684,15 +1553,15 @@ while (!done)
       else
         {
         uschar sbuf[8];
-        sbuf[0] = (uschar)((true_size >> 24) & 255);
-        sbuf[1] = (uschar)((true_size >> 16) & 255);
-        sbuf[2] = (uschar)((true_size >>  8) & 255);
-        sbuf[3] = (uschar)((true_size) & 255);
+        sbuf[0] = (true_size >> 24)  & 255;
+        sbuf[1] = (true_size >> 16)  & 255;
+        sbuf[2] = (true_size >>  8)  & 255;
+        sbuf[3] = (true_size)  & 255;
 
-        sbuf[4] = (uschar)((true_study_size >> 24) & 255);
-        sbuf[5] = (uschar)((true_study_size >> 16) & 255);
-        sbuf[6] = (uschar)((true_study_size >>  8) & 255);
-        sbuf[7] = (uschar)((true_study_size) & 255);
+        sbuf[4] = (true_study_size >> 24)  & 255;
+        sbuf[5] = (true_study_size >> 16)  & 255;
+        sbuf[6] = (true_study_size >>  8)  & 255;
+        sbuf[7] = (true_study_size)  & 255;
 
         if (fwrite(sbuf, 1, 8, f) < 8 ||
             fwrite(re, 1, true_size, f) < true_size)
@@ -1736,7 +1605,7 @@ while (!done)
     int callout_data_set = 0;
     int count, c;
     int copystrings = 0;
-    int find_match_limit = default_find_match_limit;
+    int find_match_limit = 0;
     int getstrings = 0;
     int getlist = 0;
     int gmatched = 0;
@@ -1766,7 +1635,8 @@ while (!done)
     len = 0;
     for (;;)
       {
-      if (extend_inputline(infile, buffer + len, "data> ") == NULL)
+      if (infile == stdin) printf("data> ");
+      if (extend_inputline(infile, buffer + len) == NULL)
         {
         if (len > 0) break;
         done = 1;
@@ -1834,19 +1704,9 @@ while (!done)
             {
             unsigned char buff8[8];
             int ii, utn;
-            if (use_utf8)
-              {
-              utn = ord2utf8(c, buff8);
-              for (ii = 0; ii < utn - 1; ii++) *q++ = buff8[ii];
-              c = buff8[ii];   /* Last byte */
-              }
-            else
-             {
-             if (c > 255)
-               fprintf(outfile, "** Character \\x{%x} is greater than 255 and "
-                 "UTF-8 mode is not enabled.\n"
-                 "** Truncation will probably give the wrong result.\n", c);
-             }
+            utn = ord2utf8(c, buff8);
+            for (ii = 0; ii < utn - 1; ii++) *q++ = buff8[ii];
+            c = buff8[ii];   /* Last byte */
             p = pt + 1;
             break;
             }
@@ -1976,10 +1836,7 @@ while (!done)
         continue;
 
         case 'N':
-        if ((options & PCRE_NOTEMPTY) != 0)
-          options = (options & ~PCRE_NOTEMPTY) | PCRE_NOTEMPTY_ATSTART;
-        else
-          options |= PCRE_NOTEMPTY;
+        options |= PCRE_NOTEMPTY;
         continue;
 
         case 'O':
@@ -2002,8 +1859,7 @@ while (!done)
         continue;
 
         case 'P':
-        options |= ((options & PCRE_PARTIAL_SOFT) == 0)?
-          PCRE_PARTIAL_SOFT : PCRE_PARTIAL_HARD;
+        options |= PCRE_PARTIAL;
         continue;
 
         case 'Q':
@@ -2038,10 +1894,6 @@ while (!done)
         show_malloc = 1;
         continue;
 
-        case 'Y':
-        options |= PCRE_NO_START_OPTIMIZE;
-        continue;
-
         case 'Z':
         options |= PCRE_NOTEOL;
         continue;
@@ -2064,23 +1916,6 @@ while (!done)
     *q = 0;
     len = q - dbuffer;
 
-    /* Move the data to the end of the buffer so that a read over the end of
-    the buffer will be seen by valgrind, even if it doesn't cause a crash. If
-    we are using the POSIX interface, we must include the terminating zero. */
-
-#if !defined NOPOSIX
-    if (posix || do_posix)
-      {
-      memmove(bptr + buffer_size - len - 1, bptr, len + 1);
-      bptr += buffer_size - len - 1;
-      }
-    else
-#endif
-      {
-      memmove(bptr + buffer_size - len, bptr, len);
-      bptr += buffer_size - len;
-      }
-
     if ((all_use_dfa || use_dfa) && find_match_limit)
       {
       printf("**Match limit not relevant for DFA matching: ignored\n");
@@ -2100,7 +1935,6 @@ while (!done)
         pmatch = (regmatch_t *)malloc(sizeof(regmatch_t) * use_size_offsets);
       if ((options & PCRE_NOTBOL) != 0) eflags |= REG_NOTBOL;
       if ((options & PCRE_NOTEOL) != 0) eflags |= REG_NOTEOL;
-      if ((options & PCRE_NOTEMPTY) != 0) eflags |= REG_NOTEMPTY;
 
       rc = regexec(&preg, (const char *)bptr, use_size_offsets, pmatch, eflags);
 
@@ -2156,7 +1990,7 @@ while (!done)
           {
           int workspace[1000];
           for (i = 0; i < timeitm; i++)
-            count = pcre_dfa_exec(re, extra, (char *)bptr, len, start_offset,
+            count = pcre_dfa_exec(re, NULL, (char *)bptr, len, start_offset,
               options | g_notempty, use_offsets, use_size_offsets, workspace,
               sizeof(workspace)/sizeof(int));
           }
@@ -2219,7 +2053,7 @@ while (!done)
       else if (all_use_dfa || use_dfa)
         {
         int workspace[1000];
-        count = pcre_dfa_exec(re, extra, (char *)bptr, len, start_offset,
+        count = pcre_dfa_exec(re, NULL, (char *)bptr, len, start_offset,
           options | g_notempty, use_offsets, use_size_offsets, workspace,
           sizeof(workspace)/sizeof(int));
         if (count == 0)
@@ -2374,12 +2208,11 @@ while (!done)
       else if (count == PCRE_ERROR_PARTIAL)
         {
         fprintf(outfile, "Partial match");
-        if (use_size_offsets > 1)
-          {
-          fprintf(outfile, ": ");
-          pchars(bptr + use_offsets[0], use_offsets[1] - use_offsets[0],
-            outfile);
-          }
+#if !defined NODFA
+        if ((all_use_dfa || use_dfa) && use_size_offsets > 2)
+          fprintf(outfile, ": %.*s", use_offsets[1] - use_offsets[0],
+            bptr + use_offsets[0]);
+#endif
         fprintf(outfile, "\n");
         break;  /* Out of the /g loop */
         }
@@ -2409,11 +2242,9 @@ while (!done)
             {
             int d;
             (void)pcre_config(PCRE_CONFIG_NEWLINE, &d);
-            /* Note that these values are always the ASCII ones, even in
-            EBCDIC environments. CR = 13, NL = 10. */
-            obits = (d == 13)? PCRE_NEWLINE_CR :
-                    (d == 10)? PCRE_NEWLINE_LF :
-                    (d == (13<<8 | 10))? PCRE_NEWLINE_CRLF :
+            obits = (d == '\r')? PCRE_NEWLINE_CR :
+                    (d == '\n')? PCRE_NEWLINE_LF :
+                    (d == ('\r'<<8 | '\n'))? PCRE_NEWLINE_CRLF :
                     (d == -2)? PCRE_NEWLINE_ANYCRLF :
                     (d == -1)? PCRE_NEWLINE_ANY : 0;
             }
@@ -2452,9 +2283,9 @@ while (!done)
       if (!do_g && !do_G) break;
 
       /* If we have matched an empty string, first check to see if we are at
-      the end of the subject. If so, the /g loop is over. Otherwise, mimic what
-      Perl's /g options does. This turns out to be rather cunning. First we set
-      PCRE_NOTEMPTY_ATSTART and PCRE_ANCHORED and try the match again at the
+      the end of the subject. If so, the /g loop is over. Otherwise, mimic
+      what Perl's /g options does. This turns out to be rather cunning. First
+      we set PCRE_NOTEMPTY and PCRE_ANCHORED and try the match again at the
       same point. If this fails (picked up above) we advance to the next
       character. */
 
@@ -2463,7 +2294,7 @@ while (!done)
       if (use_offsets[0] == use_offsets[1])
         {
         if (use_offsets[0] == len) break;
-        g_notempty = PCRE_NOTEMPTY_ATSTART | PCRE_ANCHORED;
+        g_notempty = PCRE_NOTEMPTY | PCRE_ANCHORED;
         }
 
       /* For /g, update the start offset, leaving the rest alone */
